
package org.jbpm.test;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.List;

import org.jbpm.api.jpdl.DecisionHandler;
import org.jbpm.api.model.OpenExecution;

public class FileChecker implements DecisionHandler{

    private String fromVariable;
    private static String IN_PROCESSING = "IN_PROCESSING";
    private static String ZIP_FILES = "ZIP_FILES";

    @Override
    public String decide(OpenExecution oe) {

        String result = null;
        Boolean inProcessing = (Boolean) oe.getVariable(IN_PROCESSING);
        if (inProcessing == null) {
            // not started so see if there are any files to start processing
            String from = "testfiles";

            File directory = new File(from);
            // get all .zip files in directory
            File[] filesInDir = directory.listFiles(new ZIPFileFilter());
            if (filesInDir != null && filesInDir.length > 0) {

                System.out.println("Processing " + filesInDir.length
                        + " files in directory \"" + from + "\"");

                List<File> files = new ArrayList<File>(filesInDir.length);
                for (File farFile : filesInDir) {
                    files.add(farFile);
                }

                File file = files.remove(0);

                System.out.println("Processing file \"" + file.getName() + "\"");

                result = "yes";
                oe.setVariable("filename", file.getName());
                oe.setVariable(IN_PROCESSING, Boolean.TRUE);
                oe.setVariable(ZIP_FILES, files);
            } else {
                System.out.println("No files found to process in directory "
                        + from);
                result = "no";
                cleanUp(oe);
            }

        } else if (inProcessing.equals(Boolean.TRUE)) {

            // in processing so check if any files left to process
            List<File> files = (List<File>) oe.getVariable(ZIP_FILES);
            if (files != null && files.size() > 0) {
                File farFile = files.remove(0);
                result = "yes";
                oe.setVariable("filename", farFile.getName());
                oe.setVariable(IN_PROCESSING, Boolean.TRUE);
                oe.setVariable(ZIP_FILES, files);
            } else {
                result = "no";
                cleanUp(oe);
            }

        } else {
            result = "no";
            cleanUp(oe);
        }

        return result;
    }

    public void setFromVariable(String fromVariable) {
        this.fromVariable = fromVariable;
    }

    private void cleanUp(OpenExecution oe) {

        oe.removeVariable(IN_PROCESSING);
        oe.removeVariable(ZIP_FILES);
    }
}

/**
 * Filter to find only ZIP files.
 */
class ZIPFileFilter implements FileFilter{

    public ZIPFileFilter() {
    }

    public boolean accept(File f) {
        return !f.isDirectory() && f.getName().toLowerCase().endsWith(".zip");
    }
}
