package org.jbpm.test;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

import org.jbpm.api.activity.ActivityExecution;

public class ProcessorActivity extends AbstractRepeatableForkActivity{

    private transient XMLEventReader xmlEventReader = null;
    private String fileName;
    private String directory;

    @Override
    protected void cleanUp(ActivityExecution ae) {
        try {
            if (xmlEventReader != null) {
                xmlEventReader.close();
            }
        } catch (XMLStreamException xse) {
        }

        if (this.fileName != null) {
            File file = new File(fileName);
            file.delete();
        }
    }

    @Override
    protected void setUp(ActivityExecution ae) {

        String filename = (String) ae.getVariable("filename");
        String fileDirectory = "testfiles";

        try {
            ZipFile farZip = new ZipFile(fileDirectory + File.separator
                    + filename);
            Enumeration<? extends ZipEntry> entries = farZip.entries();

            if (entries.hasMoreElements()) {
                ZipEntry entry = (ZipEntry) entries.nextElement();
                if (!entry.isDirectory()) {
                    System.out.println("Extracting the file :" + entry.getName());

                    this.fileName = fileDirectory + File.separator
                            + entry.getName();
                    copyInputStream(farZip.getInputStream(entry), new BufferedOutputStream(new FileOutputStream(fileName)));

                    System.out.println("Extracted the file :" + entry.getName());
                }
            }
            if (fileName != null) {
                xmlEventReader = XMLInputFactory.newInstance().createXMLEventReader(new FileInputStream(fileName));
            }
        } catch (ZipException ze) {
        } catch (IOException io) {
        } catch (XMLStreamException xse) {
        }
    }

    protected void copyInputStream(InputStream in, OutputStream out) throws IOException {

        byte[] buffer = new byte[1024];
        int len;

        while ((len = in.read(buffer)) >= 0) {
            out.write(buffer, 0, len);
        }

        in.close();
        out.close();
    }

    @Override
    protected Map<String, ?> getVariablesForRepeat() {
        try {
            Map<String, String> map = new HashMap<String, String>();
            boolean done = false;

            while (!done && xmlEventReader.hasNext()) {
                XMLEvent event = xmlEventReader.nextEvent();
                int eventType = event.getEventType();

                switch (eventType) {
                case XMLEvent.START_ELEMENT: {
                    StartElement element = event.asStartElement();
                    if (element.getName().getLocalPart().toLowerCase().equals("transaction")) {
                        Iterator iterator = element.getAttributes();
                        while (iterator.hasNext()) {
                            Attribute attribute = (Attribute) iterator.next();
                            map.put(attribute.getName().getLocalPart(), attribute.getValue());
                        }
                    }
                    break;
                }
                case XMLEvent.END_ELEMENT: {
                    
                    EndElement element = event.asEndElement();
                    if (element.getName().getLocalPart().toLowerCase().equals("transaction")) {
                        done = true;
                    }
                    break;
                }
                }
            }
            return map;
        } catch (XMLStreamException xse) {
            return null;
        }
    }

    @Override
    protected RepeatInfo shouldRepeat() {
        RepeatInfo result = RepeatInfo.REPEAT;
        try {
            if (xmlEventReader != null && xmlEventReader.hasNext()) {
                XMLEvent event = xmlEventReader.peek();
                int eventType = event.getEventType();
                while ((eventType == XMLEvent.CHARACTERS) // skip whitespace
                        || (eventType == XMLEvent.CDATA) // skip whitespace
                        || eventType == XMLEvent.SPACE
                        || eventType == XMLEvent.PROCESSING_INSTRUCTION
                        || eventType == XMLEvent.COMMENT
                        || eventType == XMLEvent.START_DOCUMENT) {
                    xmlEventReader.nextEvent();
                    event = xmlEventReader.peek();
                    eventType = event.getEventType();
                }
                if (eventType == XMLEvent.END_DOCUMENT) {
                    result = RepeatInfo.NOT;
                } else if (event.isEndElement()
                        && ((EndElement) event).getName().getLocalPart().equalsIgnoreCase("Report")) {

                    result = RepeatInfo.NOT;
                }
            }

            return result;
        } catch (XMLStreamException xse) {
            return null;
        }
    }
    
    public void setDirectory(String directory) {
        this.directory = directory;
    }
}